/*
   Simple glass tube macro. See test scene for parameters explanation.
*/

#macro Glass_Tube(
 g_height,
 g_mn_rad,
 g_mj_rad,
 g_thick,
 g_bott_thick,
 g_m_glass,
 g_liq_lev,
 g_m_liquid
)

merge{
 // glass tube
 union{
  difference{
   cone{
    <0,-g_height*.5+g_thick*.5,0>,g_mn_rad
    <0, g_height*.5,0>,g_mj_rad
   }
   cone{
    <0,-g_height*.5+g_bott_thick,0>,g_mn_rad-g_thick
    <0, g_height*.5+.001,0>,g_mj_rad-g_thick+.001
   }
   bounded_by{
    cone{
     <0,-g_height*.5+g_thick*.5,0>,g_mn_rad
     <0, g_height*.5,0>,g_mj_rad
     scale 1.01
    }
   }
  }
  torus{
   g_mj_rad-g_thick*.5,g_thick*.5
   translate g_height*.5*y
  }
  torus{
   g_mn_rad-g_thick*.5,g_thick*.5
   translate (-g_height*.5+g_thick*.5)*y
  }
  material{g_m_glass}
 }
 #if (g_liq_lev>0)
 // liquid
 intersection{
  cone{
   <0,-g_height*.5+g_bott_thick+.001,0>,g_mn_rad-g_thick-.001
   <0, g_height*.5,0>,g_mj_rad-g_thick-.001
  }
  plane{y,-g_height*.5+g_liq_lev}
  material{g_m_liquid}
 }
 #end
 bounded_by{cylinder{<0,-g_height*.5-g_thick,0>,<0,g_height*.5+g_thick,0>,g_mj_rad+g_thick}}
}

#end


// *** drops loops ***
#if (trace_drops)

/* +++ textura del agua de condensacion +++ */
#declare t_agua=
texture {
 Glass
 finish {
    reflection .1
    ior 1.3
    caustics 0.9
 }
}
 
// *** pseudo-charco en el suelo, alrededor del tubo ***
#declare num_gotas=50;
#declare cnt_gotas=1;
#declare R1=seed(0);
#declare R2=seed(1357);
#declare signoy=1;
#declare signox=1;
#declare radio=3;
#declare margen=2;
#declare charco=
blob{ threshold 0.5
#while (cnt_gotas<=num_gotas)
  sphere{
   <0,0,0>,0.5,1
   scale <0.5+rand(R2),.5,0.5+rand(R1)>
   translate (-margen-radio*rand(R1))*z
   rotate 360*rand(R2)*y
  }
  #declare cnt_gotas=cnt_gotas+1;
#end
texture{
#if (trace_glass=0)
 pigment{Red}
#else
 t_agua
#end
}
}

// *** gotitas en la superficie del tubo, alrededor ***
#declare num_gotas=400;
#declare cnt_gotas=0;
#declare R1=seed(0);
#declare R2=seed(1357);
#declare radio=2.99;
#declare gotitas=
difference{
 blob{ threshold 0.5
 #while (cnt_gotas<=num_gotas)
   sphere{
    <0,0,0>,0.2,1
    scale <0.5+0.5*rand(R1),0.5+rand(R1),0.4>
    translate -radio*z
    rotate 360*rand(R1)*y
    translate 13*rand(R1)*y
   }
   #declare cnt_gotas=cnt_gotas+1;
 #end
 // segunda pasada
 #declare cnt_gotas=0;
 #while (cnt_gotas<=num_gotas)
   sphere{
    <0,0,0>,0.22,1
    scale <0.5+0.5*rand(R1),0.5+rand(R1),0.4>
    translate -radio*z
    rotate 360*rand(R1)*y
    translate 9*rand(R1)*y
   }
   #declare cnt_gotas=cnt_gotas+1;
 #end
 }
 cylinder{<0,0,0>,<0,13.2,0>,3}
 texture{
 #if (trace_glass=0)
  pigment{Red}
 #else
  t_agua
 #end
}
} 

// *** burbujas de la cerveza ***
#declare num_gotas=300;
#declare cnt_gotas=0;
#declare R1=seed(0);
#declare R2=seed(1357);
#declare radio=3.01;
#declare burbujas=
blob{ threshold 0.5
 #while (cnt_gotas<=num_gotas)
  #declare rx=(radio*cos(cnt_gotas)*.95)*rand(R1);
  #declare rz=(radio*sin(cnt_gotas)*.95)*rand(R1);
  sphere{<rx,0,rz>,0.1,1
   translate (1.5+7.6*rand(R1))*y
  }
  #declare cnt_gotas=cnt_gotas+1;
 #end
 #if (trace_glass=0)
  texture{pigment{Red}}
 #else
  texture{Glass2}
 #end
}


// *** espuma de cerveza ***
#declare t_espuma=
texture{
 pigment{
  crackle
  color_map{
   [0 White*.7+Goldenrod*.2]
   [1 Clear]
  }
 }
 normal{crackle}
 finish{reflection .2}
 scale .1
}
#declare num_gotas=800;
#declare cnt_gotas=0;
#declare R1=seed(0);
#declare R2=seed(1357);
#declare radio=3.2;
#declare espuma=
union{
 // espuma propia
 intersection{
  blob{ threshold 0.5
   #while (cnt_gotas<=num_gotas)
    #declare rx=(radio*cos(cnt_gotas))*rand(R1);
    #declare rz=(radio*sin(cnt_gotas))*rand(R1);
    sphere{<rx,0,rz>,0.2+0.3*rand(R1),1
     translate (8.8+1.1*rand(R1))*y
    }
    #declare cnt_gotas=cnt_gotas+1;
   #end
  }
  cylinder{<0,9,0>,<0,12,0>,2.8}
  texture{t_espuma}
 }
 // rastro en la cara interior    
 #declare p_espuma=
 pigment{
  wrinkles
  color_map{
   [0.0 Clear]
   [0.2 Clear]
   [0.4 White*.7+Goldenrod*.3 filter 0.5]
   [1.0 White*.7+Goldenrod*.3 filter 0.7]
  }
  scale .1
 }
 difference{
  cylinder{<0,9,0>,<0,13,0>,2.8}
  cylinder{<0,8.9,0>,<0,13.1,0>,2.79}
  texture{
   pigment{
    gradient y
    turbulence 0.2
    pigment_map{
     [0.0 Clear]
     [0.3 Clear]
     [0.6 p_espuma]
     [0.65 Clear]
     [0.9 p_espuma]
     [1.0 Clear]
    }
    scale 4
    translate 0*y
   }
   finish{Luminous}
  }
 }
}
#end
